using System.Collections.Generic;
using System.Windows.Forms;
using JetBrains.ReSharper.OptionsUI;

namespace ReSharper.IntroduceStringConstant {
  [OptionsPage(
   "PowerToys.IntroduceStringConstant",
    "Introduce String Constant",
    null,
    ParentId = "ContextActionsPage",
   InsertionPosition = PageInsertionPosition.LAST)]
  public partial class OptionPage : UserControl, IOptionsPage {
    #region Fields

    readonly IOptionsUI _optionsUI;

    #endregion

    #region Constructor

    /// <summary>
    /// Initializes a new instance of the <see cref="OptionPage"/> class.
    /// </summary>
    /// <param name="optionsUI">The options UI.</param>
    public OptionPage(IOptionsUI optionsUI) {
      _optionsUI = optionsUI;
      InitializeComponent();
    }

    #endregion

    #region Public properties

    /// <summary>
    /// Control to be shown as page
    /// </summary>
    /// <value></value>
    public Control Control {
      get {
        return this;
      }
    }

    #endregion

    #region Public methods

    /// <summary>
    /// Initialize the UI of the page
    /// </summary>
    public void InitializeUI() {
      Settings settings = Settings.Instance;

      List<string> classNames = settings.ClassNames;

      foreach(string className in classNames){
        methodsGridView.Rows.Add(new string[]{className});
      }

      rbReplaceWithUnderscore.Checked = settings.ReplaceSpacesMode == 0;
      rbReplaceWithNothing.Checked = settings.ReplaceSpacesMode == 1;

      rbUppercase.Checked = settings.TransformIdentifierMode == 0;
      rbCamelCase.Checked = settings.TransformIdentifierMode == 1;
      rbLeave.Checked = settings.TransformIdentifierMode == 2;

      cbGenerateXmlComment.Checked = settings.GenerateXmlComment;
    }

    /// <summary>
    /// Invoked when this page is selected/unselected in the tree
    /// </summary>
    /// <param name="activated">true, when page is selected; false, when page is unselected</param>
    public void OnActivated(bool activated) {
    }

    /// <summary>
    /// Check if the settings on the page are consistent, and page could be closed
    /// </summary>
    /// <returns><c>true</c> if page data is consistent</returns>
    public bool ValidatePage() {
      return true;
    }

    /// <summary>
    /// Invoked when OK button in the options dialog is pressed
    /// If the page returns <c>false</c>, the the options dialog won't be closed, and focus
    /// will be put into this page
    /// </summary>
    /// <returns></returns>
    public bool OnOk() {
      Settings settings = Settings.Instance;

      List<string> classNames = settings.ClassNames;
      classNames.Clear();

      DataGridViewRowCollection rows = methodsGridView.Rows;

      foreach(DataGridViewRow row in rows){
        object value = row.Cells[0].Value;

        if(value != null){
          classNames.Add(value.ToString());
        }
      }

      if (rbReplaceWithUnderscore.Checked){
        settings.ReplaceSpacesMode = 0;
      }
      else{
        settings.ReplaceSpacesMode = 1;
      }

      if (rbUppercase.Checked){
        settings.TransformIdentifierMode = 0;
      }
      else if (rbCamelCase.Checked){
        settings.TransformIdentifierMode = 1;
      }
      else{
        settings.TransformIdentifierMode = 2;
      }

      settings.GenerateXmlComment = cbGenerateXmlComment.Checked;

      JetBrains.ReSharper.Daemon.Daemon.GetInstance(_optionsUI.Solution).InvalidateAll();

      return true;
    }

    #endregion
  }
}
