using JetBrains.ReSharper.Daemon;
using JetBrains.ReSharper.Psi.CSharp.Tree;
using JetBrains.ReSharper.Psi.Tree;
using ReSharper.Framework;

namespace ReSharper.ParameterAsserts {
  /// <summary>
  /// Represents a GenerateAssertionsElementProcessor.
  /// </summary>
  [DaemonStage]
  public class Highlighter : HighlighterBase {
    #region Constructor

    /// <summary>
    /// Initializes a new instance of the <see cref="Highlighter"/> class.
    /// </summary>
    public Highlighter() {
    }

    /// <summary>
    /// Initializes a new instance of the <see cref="Highlighter"/> class.
    /// </summary>
    /// <param name="daemonProcess">The daemon process.</param>
    public Highlighter(IDaemonProcess daemonProcess) : base(daemonProcess) {
    }

    #endregion

    #region Protected methods

    /// <summary>
    /// Run after the interior has run.
    /// </summary>
    /// <param name="element">The element.</param>
    protected override void ProcessElement(IElement element) {
      IParameterDeclaration parameterDeclaration = element as IParameterDeclaration;

      if(parameterDeclaration != null){
        ProcessParameterDeclaration(parameterDeclaration);
        return;
      }

      IFunctionDeclaration functionDeclaration = element as IFunctionDeclaration;
      if(functionDeclaration != null){
        ProcessFunctionDeclaration(functionDeclaration);
        return;
      }
    }

    #endregion

    #region Private methods

    /// <summary>
    /// Processes the function declaration.
    /// </summary>
    /// <param name="parameterDeclaration">The declaration.</param>
    void ProcessParameterDeclaration(IParameterDeclaration parameterDeclaration) {
      IFunctionDeclaration functionDeclaration = parameterDeclaration.GetContainingTypeMemberDeclaration() as IFunctionDeclaration;
      if(functionDeclaration == null){
        return;
      }

      if(functionDeclaration.Body == null){
        return;
      }

      if(ParameterAssert.IsParameterMissingAssertion(functionDeclaration, parameterDeclaration)) {
        string message = string.Format("The parameter '{0}' is not asserted", parameterDeclaration.DeclaredName);

        AddHighlighting(parameterDeclaration.GetNameDocumentRange(), message);
      }
    }

    /// <summary>
    /// Processes the function declaration.
    /// </summary>
    /// <param name="functionDeclaration">The declaration.</param>
    void ProcessFunctionDeclaration(IFunctionDeclaration functionDeclaration) {
      if(functionDeclaration.Body == null) {
        return;
      }

      if(ParameterAssert.IsFunctionMissingAssertions(functionDeclaration)) {
        AddHighlighting(functionDeclaration.GetNameDocumentRange(), "The parameter 'value' is not asserted");
      }
    }

    #endregion  
  }
}