using System.Collections.Specialized;
using System.Windows.Forms;
using JetBrains.ReSharper.OptionsUI;

namespace ReSharper.ParameterAsserts {
  [OptionsPage(
   "PowerToys.ParameterAssert",
    "Parameter Asserts",
    null,
    ParentId = "HighlightingPage",
   InsertionPosition = PageInsertionPosition.LAST)]
  public partial class OptionsPage : UserControl, IOptionsPage {
    #region Fields

    readonly IOptionsUI _optionsUI;

    #endregion

    #region Constructor

    /// <summary>
    /// Initializes a new instance of the <see cref="OptionsPage"/> class.
    /// </summary>
    /// <param name="optionsUI">The options UI.</param>
    public OptionsPage(IOptionsUI optionsUI) {
      _optionsUI = optionsUI;
      InitializeComponent();
    }

    #endregion

    #region Public properties

    /// <summary>
    /// Control to be shown as page
    /// </summary>
    /// <value></value>
    public Control Control {
      get {
        return this;
      }
    }

    #endregion

    #region Public methods

    /// <summary>
    /// Initialize the UI of the page
    /// </summary>
    public void InitializeUI() {
      NameValueCollection typeAssertions = Settings.Instance.TypeAssertions;

      foreach(string key in typeAssertions){
        typeAssertionsGridView.Rows.Add(new string[]{key, typeAssertions[key]});
      }

      StringCollection methods = Settings.Instance.Methods;

      foreach(string method in methods){
        methodsGridView.Rows.Add(new string[]{method});
      }

      PublicMethods.Checked = Settings.Instance.PublicMethods;
      InternalMethods.Checked = Settings.Instance.InternalMethods;
      ProtectedMethods.Checked = Settings.Instance.ProtectedMethods;
      PrivateMethods.Checked = Settings.Instance.PrivateMethods;

      OmitConstructors.Checked = Settings.Instance.OmitBaseConstructors;
      OmitMethods.Checked = Settings.Instance.OmitBaseMethods;
    }

    /// <summary>
    /// Invoked when this page is selected/unselected in the tree
    /// </summary>
    /// <param name="activated">true, when page is selected; false, when page is unselected</param>
    public void OnActivated(bool activated) {
    }

    /// <summary>
    /// Check if the settings on the page are consistent, and page could be closed
    /// </summary>
    /// <returns><c>true</c> if page data is consistent</returns>
    public bool ValidatePage() {
      return true;
    }

    /// <summary>
    /// Invoked when OK button in the options dialog is pressed
    /// If the page returns <c>false</c>, the the options dialog won't be closed, and focus
    /// will be put into this page
    /// </summary>
    /// <returns></returns>
    public bool OnOk() {
      NameValueCollection typeAssertions = Settings.Instance.TypeAssertions;
      typeAssertions.Clear();

      DataGridViewRowCollection rows = typeAssertionsGridView.Rows;

      foreach(DataGridViewRow row in rows){
        object key = row.Cells[0].Value;
        object value = row.Cells[1].Value;

        if (key != null && value != null){
          typeAssertions.Add(key.ToString(), value.ToString());
        }
      }

      StringCollection methods = Settings.Instance.Methods;
      methods.Clear();

      rows = methodsGridView.Rows;

      foreach(DataGridViewRow row in rows){
        object value = row.Cells[0].Value;

        if(value != null){
          methods.Add(value.ToString());
        }
      }

      Settings.Instance.OmitBaseConstructors = OmitConstructors.Checked;
      Settings.Instance.OmitBaseMethods = OmitMethods.Checked;

      Settings.Instance.PublicMethods = PublicMethods.Checked;
      Settings.Instance.InternalMethods = InternalMethods.Checked;
      Settings.Instance.ProtectedMethods = ProtectedMethods.Checked;
      Settings.Instance.PrivateMethods = PrivateMethods.Checked;

      JetBrains.ReSharper.Daemon.Daemon.GetInstance(_optionsUI.Solution).InvalidateAll();

      return true;
    }

    #endregion
  }
}
