using JetBrains.ReSharper.Daemon;
using JetBrains.ReSharper.ProjectModel;
using JetBrains.ReSharper.Psi.CSharp;
using JetBrains.ReSharper.Psi.CSharp.Tree;
using JetBrains.ReSharper.Psi.Parsing;
using JetBrains.ReSharper.Psi.Tree;
using JetBrains.ReSharper.TextControl;
using ReSharper.Framework;

namespace ReSharper.StringEmpty {
  /// <summary>
  /// Represents the Context Action.
  /// </summary>
  [ContextAction(Name="StringEmpty")]
  public class ContextAction : ContextActionBase {
    #region Constructor

    /// <summary>
    /// Initializes a new instance of the <see cref="ContextAction"/> class.
    /// </summary>
    /// <param name="solution">The solution.</param>
    /// <param name="textControl">The text control.</param>
    public ContextAction(ISolution solution, ITextControl textControl) : base(solution, textControl) {
    }

    #endregion

    #region Protected methods

    /// <summary>
    /// Gets the text.
    /// </summary>
    /// <value>The text.</value>
    protected override string GetText() {
      return "Convert to string.Empty";
    }

    /// <summary>
    /// Executes this instance.
    /// </summary>
    /// <param name="element">The element.</param>
    protected override void Execute(IElement element) {
      Modify(delegate { ConvertToStringEmpty(element); });
    }

    /// <summary>
    /// Determines whether this instance is available.
    /// </summary>
    /// <param name="element">The element.</param>
    /// <returns>
    /// 	<c>true</c> if this instance is available; otherwise, <c>false</c>.
    /// </returns>
    protected override bool IsAvailable(IElement element) {
      ITokenNode tokenType = element as ITokenNode;
      if(tokenType == null) {
        return false;
      }

      TokenNodeType type = tokenType.GetTokenType();

      if(!type.IsStringLiteral) {
        return false;
      }

      if(element.GetText() == "\"\"") {
        return true;
      }

      return false;
    }

    #endregion

    #region Private methods

    /// <summary>
    /// Generates the function assert statements.
    /// </summary>
    /// <param name="element">The element.</param>
    void ConvertToStringEmpty(IElement element) {
      ITreeNode treeNode = element as ITreeNode;
      if(treeNode == null){
        return;
      }

      ICSharpExpression expression = treeNode.Parent as ICSharpExpression;
      if(expression == null){
        return;
      }

      CSharpElementFactory factory = CSharpElementFactory.GetInstance(Solution);
      if(factory == null){
        return;
      }

      ICSharpExpression stringEmptyExpression = factory.CreateExpression("string.Empty");

      expression.ReplaceBy(stringEmptyExpression);
    }

    #endregion
  }
}
  