using JetBrains.ReSharper.Daemon;
using JetBrains.ReSharper.ProjectModel;
using JetBrains.ReSharper.Psi;
using JetBrains.ReSharper.Psi.CSharp.Tree;
using JetBrains.ReSharper.Psi.Tree;
using JetBrains.ReSharper.Util;
using ReSharper.Framework;

namespace ReSharper.SynchronizeNamespaceWithFilePath {
  /// <summary>
  /// Represents a GenerateAssertionsElementProcessor.
  /// </summary>
  [DaemonStage]
  public class Highlighter : HighlighterBase {
    #region Constructor

    /// <summary>
    /// Initializes a new instance of the <see cref="Highlighter"/> class.
    /// </summary>
    public Highlighter() {
    }

    /// <summary>
    /// Initializes a new instance of the <see cref="Highlighter"/> class.
    /// </summary>
    /// <param name="daemonProcess">The daemon process.</param>
    public Highlighter(IDaemonProcess daemonProcess) : base(daemonProcess) {
    }

    #endregion

    #region Visitor methods

    /// <summary>
    /// Run after the interior has run.
    /// </summary>
    /// <param name="element">The element.</param>
    protected override void ProcessElement(IElement element) {
      bool isAvailable = ContextAction.IsAvailable(DaemonProcess.Solution, DaemonProcess.ProjectFile, element);

      if(!isAvailable){
        return;
      }

      ICSharpNamespaceDeclarationNode namespaceDeclarationNode = element as ICSharpNamespaceDeclarationNode;
      if(namespaceDeclarationNode == null) {
        return;
      }

      IProject project = DaemonProcess.ProjectFile.GetProject();
      if(project == null) {
        return;
      }

      FileSystemPath filePath = ContextAction.GetFilePath(project, DaemonProcess.ProjectFile);

      string message = string.Format("The namespace \"{0}\" does not match the folder \"\\{1}\".", namespaceDeclarationNode.QualifiedName, filePath);

      TextRange textRange = new TextRange();
      DocumentRange documentRange;

      IOwnerQualificationNode qualificationNode = namespaceDeclarationNode.NamespaceQualification;
      if(qualificationNode != null){
        documentRange = qualificationNode.GetDocumentRange();
        textRange = documentRange.TextRange;
      }

      ICSharpIdentifierNode nameIdentifier = namespaceDeclarationNode.NameIdentifier;
      if(nameIdentifier == null){
        return;  
      }

      documentRange = nameIdentifier.GetDocumentRange();
      if(textRange.IsEmpty) {
        textRange = documentRange.TextRange;
      } else{
        textRange = textRange.Join(documentRange.TextRange);
      }

      documentRange = new DocumentRange(documentRange.Document, textRange);

      AddHighlighting(documentRange, message);
    }

    #endregion
  }
}